
*Sanction episodes taken from TIES (1971-2000)

We break each episode into three ten-year periods (1971-1980, 1981-1990, and 1991-2000).  The original episode or case included in TIES has a coded outcome: Back Down, Acquiesce, and Sanction in a particular year.  If that year is in 1975 with Sanction, then that year is included as an observation for time period one.     

In addition to Back Down, Acquiesce, and Sanction outcomes included in TIES, we add Status Quo outcomes to each case.  Specifically, if one of the original 3 outcomes occurred between 1971-1980, then Status Quo is coded as the outcome for 1981-1990, and 1991-2000 for the dyad.  If the original year is 1975 in the episode, then three observations are included, one for 1975 (Sanction), another in 1981 (Status Quo), and the third in 1991 Status Quo))*

*We designate sender (primary sender/sender 1) as state 1.  Target (primary target) is designated as state 2*

gen ccode1 = sender1

gen ccode2 = targetstate

*In cases where one of the actors was an institution, we use the member with the highest lagged value of GDP (from UN data) among member states as the main actor in the episode*

*We then drop all duplicates*

duplicates drop ccode1 ccode2 year, force

*We then add regressors, beginning with UN trade data.  All regressors are lagged values.  For instance, if episode began in 1971, then regressors for that observation are values from 1970.  For other observations on that episode, they would be as follows: 1981-1990 would use values from 1980, and 1991-2000 would use values from 1990*

*For UN trade data, we first convert UN country codes into COW country codes for compatibility before merging with TIES.  

*Before merging, we make the data compatible in terms of lagged regressors*
gen laggedUNvalue=UNvalue[_n-1]

*Now, we merge the UN trade data with TIES, merging dyadic data and sender and target economic data*

"insert TIES file extension" merge ccode1 ccode2 year using "insert the updated UN trade data file extension"  

*We then drop cases with missing data*

drop if laggedvalue==.

*save* 

*We now add additional regressors using EUGene*

*Additional regressors are extracted from EUGENE (generate a EUGene dataset in dyad form from 1970-2000 with Polity IV scores, contiguity data, alliance data, and capability scores) and matched with state codes from TIES)

*save*

*Before merging with main dataset, we make the data compatible in terms of lagged regressors*

*Examples*

gen laggedPolity=Polity[_n-1]
gen laggedContiguity=Contig[_n-1]
gen laggedAlliance=Alliance[_n-1]
gen laggedCap_1=Cap_1[_n-1]
gen laggedCap2=Cap_2[_n-1]

*Now, we merge the TIES/UN Trade data with EUGene dataset*

"insert saved TIES/UN file extension" merge ccode1 ccode2 year using "insert the EUGene dataset file extension"  

*We then drop cases with missing data*

drop if laggedPolity==.
drop if laggedContiguity==.
drop if laggedAlliance==.
drop if laggedCap_1==.
drop if laggedCap_2==.

*We then modify variables to create proper controls outlined by previous research*

*We create an alliance dummy variable*

gen ModifiedAlliance=0
replace ModifiedAlliance=1 if laggedAlliance<4

*We create a natural log of the capability ratio variable*

gen ModifiedCapRatio=ln(laggedCap_1/laggedCap_2) 

*We create two trade dependence variables*

gen SenderDep=(flow1+flow2)/ UNgdp1

gen TargetDep=(flow1+flow2)/ UNgdp2

sort ccode1 ccode2 year


