Module global_variables
	integer mSA,mVA,maA,mWA,sVVA,sVaA,sVWA,saWA,sWWA,mCB,mVB,mWB,sVVB,sVWB
	real g_pset(14)
end Module

subroutine get_cdfbvn(x,y,rhom,p)
USE Bivariate_Trivariate_Normal
Implicit none
real  x,y,rhom
real, INTENT(OUT):: p
	p = bvn(-x,-y,rhom)
End subroutine

subroutine solve_eq_cPBE(n,psets,tol,pfrc)
USE global_variables
Implicit none
integer n
real psets(14,n), tol
real, INTENT(OUT):: pfrc(4,n)
integer i
real pf, p3(3), prior
real, external:: eq_pfrc_cPBE_func, zeroin
integer, external :: isOK_pset

!open (3, FILE='logf.txt',FORM ='FORMATTED')
	
	do i=1,n
		g_pset = psets(:,i)
		if (isOK_pset(g_pset)<0) then
			pfrc = -1.0
			EXIT
		endif

		pf = zeroin(eq_pfrc_cPBE_func, 0.0, 1.0, tol, tol)

		call eq_pfrc_cPBE(pf,g_pset,p3)
		pfrc(1,i) = pf
		pfrc(2,i) = p3(2)
		pfrc(3,i) = p3(3)		
		call get_pf_prior_cPBE(g_pset,prior)
		pfrc(4,i) = prior
	end do
!close(3)

End subroutine

Function eq_pfrc_cPBE_func(pf)
USE global_variables
Implicit none
real pf, temp(3)
real eq_pfrc_cPBE_func
	call eq_pfrc_cPBE(pf,g_pset,temp)
	eq_pfrc_cPBE_func = temp(1)
End Function

subroutine eq_pfrc_cPBE(pf,pset,df)
USE global_variables
Implicit none
real  pf,pset(12)
real, INTENT(OUT):: df(3)
real  pr,pc,pfn
real pc2

	call get_pr_cPBE(pf,pset,pr)
	if (pr==0.0) then
		if ( pset(mVA) > pset(mSA) ) then
			pc = 1.0
			call get_pf_prior_cPBE(pset,pfn)
		elseif ( pset(mVA) < pset(mSA) ) then
			pc = 0.0
			pfn = 0.0
		else
			pc = 0.5
			call get_pf_prior_cPBE(pset,pfn)
			pfn = pfn / 2.0
		endif
	else	
		call get_pc_cPBE(pr,pset,pc2)
		call get_pf_numerator_cPBE(pr,pset,pfn)
		pc = pc2 + pfn
	endif

!	df(1) = pf*pc - pfn
	df(1) = pf - pfn/pc
	df(2) = pr
	df(3) = pc
	
End subroutine

subroutine get_pr_cPBE(pf,pset,pr)
USE global_variables
USE Bivariate_Trivariate_Normal
Implicit none
real  pf,pset(12)
real, INTENT(OUT):: pr
real  Xmean, Xstd
	if (pf==0.0) then
		if ( pset(mCB) < pset(mVB) ) then
			pr = 1.0
		elseif ( pset(mCB) > pset(mVB) ) then
			pr = 0.0
		else
			pr = 0.5
		endif
	else
		Xmean = pf*pset(mWB) + (1-pf)* pset(mVB) - pset(mCB)
		Xstd = (pf**2 + (1-pf)**2*abs(pset(sVVB)) + 2*pf*(1-pf)*pset(sVWB))**0.5
		pr =  phid(Xmean / Xstd)
	endif

End subroutine

subroutine get_pc_cPBE(pr,pset,pc2)
USE global_variables
USE Bivariate_Trivariate_Normal
Implicit none
real  pr,pset(12)
real, INTENT(OUT):: pc2
!real  Ymean, Ystd, Zmean, Zstd, YZcorr

real  Umean, Ustd, Vmean, Vstd, UVcorr   !, pf_temp

	Umean = pset(maA) - pset(mWA)
	Ustd = ( 1 + pset(sWWA) - 2*pset(saWA) )**0.5
	Vmean = (1-pr)*pset(mVA) + pr*pset(maA) - pset(mSA)
	Vstd = ( (1-pr)**2*pset(sVVA) + pr**2 + 2*(1-pr)*pr*pset(sVaA) )**0.5
	UVcorr = (1-pr)*(pset(sVaA) - pset(sVWA)) + pr*( 1 - pset(saWA) )
	UVcorr = UVcorr / (Ustd*Vstd)
	
!	pf_temp = bvn(-Umean/Ustd,-Vmean/Vstd,UVcorr)
!	if ((pc==0.0) .AND. (pf_temp>0)) then 
!		pf = 1000	
!	elseif ((pc==0.0) .AND. (pf_temp<0)) then 
!		pf = -1000
!	else
		pc2 = bvn(-Umean/Ustd,-Vmean/Vstd,UVcorr)


! original version
!	Ymean = pset(mSA) - (1-pr)*pset(mVA) - pr*pset(maA)
!	Ystd = ( (1-pr)**2*pset(sVVA) + pr**2 + 2*(1-pr)*pr*pset(sVaA) )**0.5
!	Zmean = pset(mSA) -(1-pr)*pset(mVA) - pr*pset(mWA)
!	Zstd = ( (1-pr)**2*pset(sVVA) + pr**2*pset(sWWA) + 2*(1-pr)*pr*pset(sVWA) )**0.5
!	YZcorr = (1-pr)**2*pset(sVVA) +(1-pr)*pr*(pset(sVWA)+pset(sVaA))+pr**2*pset(saWA)
!	YZcorr = YZcorr / (Ystd*Zstd)


!	pc = 1- bvn(-Ymean/Ystd,-Zmean/Zstd,YZcorr)  ! note : -, -, +

End subroutine

subroutine get_pf_numerator_cPBE(pr,pset,pfn)
USE global_variables
USE Bivariate_Trivariate_Normal
Implicit none
real  pr,pset(12)
real, INTENT(OUT):: pfn
real  Umean, Ustd, Vmean, Vstd, UVcorr   !, pf_temp

	Umean = pset(mWA) - pset(maA)
	Ustd = ( pset(sWWA) + 1 - 2*pset(saWA) )**0.5
	Vmean = (1-pr)*pset(mVA) + pr*pset(mWA) - pset(mSA)
	Vstd = ( (1-pr)**2*pset(sVVA) + pr**2*pset(sWWA) + 2*(1-pr)*pr*pset(sVWA) )**0.5
	UVcorr = (1-pr)*(pset(sVWA) - pset(sVaA)) + pr*( pset(sWWA) - pset(saWA) )
	UVcorr = UVcorr / (Ustd*Vstd)
	
!	pf_temp = bvn(-Umean/Ustd,-Vmean/Vstd,UVcorr)
!	if ((pc==0.0) .AND. (pf_temp>0)) then 
!		pf = 1000	
!	elseif ((pc==0.0) .AND. (pf_temp<0)) then 
!		pf = -1000
!	else
		pfn = bvn(-Umean/Ustd,-Vmean/Vstd,UVcorr)
!		pf = pf_temp
!	end if
	
End subroutine

subroutine get_pf_prior_cPBE(pset,prior)
USE global_variables
USE Bivariate_Trivariate_Normal
Implicit none
real  pset(12)
real, INTENT(OUT):: prior
real  Umean, Ustd

	Umean = pset(mWA) - pset(maA)
	Ustd = ( pset(sWWA) + 1 - 2*pset(saWA) )**0.5
	prior = phid( Umean / Ustd )
End subroutine

subroutine phid_test(z,p)	! normal cdf
USE Bivariate_Trivariate_Normal
Implicit none
real  z
real, INTENT(OUT):: p
	p = phid(z)
	
end subroutine

Function isOK_pset(pset)
USE global_variables
USE eigen
Implicit none
real pset(14)
integer isOK_pset, flag 
real V(3,3), eigenv(3), trace, sume
integer ier

    flag=1
    if ((pset(sVVA)<0) .OR. (pset(sWWA)<0) .OR. (pset(sVVB)<0)) then
        flag=-1
    else
        if (pset(sVVA)==0) then
            if ( (pset(sVaA)/=0.0) .OR. (pset(sVWA)/=0.0) ) flag=-2
        else
            if (abs(pset(sVaA)/pset(sVVA))**0.5 > 1) flag=-3
            if (abs(pset(sVWA)/(pset(sVVA)*pset(sWWA))**0.5) > 1.0) flag=-4
        endif
        
        if (abs(pset(saWA)/(pset(sWWA))**0.5) > 1) flag=-5
        if (abs(pset(sVWB)/(pset(sVVB))**0.5) > 1) flag=-6
        
	endif

	V = reshape( source = (/ pset(sVVA), pset(sVaA), pset(sVWA), &
							pset(sVaA),        1.0, pset(saWA), &
							pset(sVWA), pset(saWA), pset(sWWA) /),&
				 shape = (/3,3/))
	call kaiser(V,3, 3, eigenv, trace, sume, ier)
!write (3,*), eigenv

    if (any(eigenv<0)) flag = -7     
    isOK_pset = flag
End Function

subroutine init_cPBE_dll()
USE global_variables
Implicit none

	mSA=1
	mVA=2
	maA=3
	mWA=4
	sVVA=5
	sVaA=6
	sVWA=7
	saWA=8
	sWWA=9
	mCB=10
	mVB=11
	mWB=12
	sVVB=13
	sVWB=14
end subroutine
